%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This Code Produces Figure 1 for
% Monetary and Fiscal Policy with Finite Lives
% By Roger E. A. Farmer abd Pawel Zabczyk
% This Version: August 2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
om = NaN(1,3);
B  = NaN(1,3);



Beta = 1;
Lambda = 1;

B = Beta.^([0 1 2]);
ET = 1/3.5;

om(2) = 2.33;
om1 = @(om2) 1 + (Beta/Lambda)*Lambda^ET/(1+Beta/Lambda) -...
    (Beta/Lambda)*om(2)/(1+Beta/Lambda);
om3 = @(om2) Lambda^ET/(1+Beta/Lambda) + Lambda^(2*ET) - ...
    om(2)/(1+Beta/Lambda);
om(1) = om1(om(2));
om(3) = om3(om(2));

h = @(Q) om(1) + om(2).*Q + om(3).*Q.^2;
psi = @(Q) 1 + B(2).^(ET).*Q.^(1-ET)...
    + B(3).^(ET).*Q.^(2*(1-ET));
wt = @(Q) [Q.^0; B(2).^(ET).*Q.^(-ET)...
    ;B(3).^(ET).*Q.^(-2*ET)];

C = @(Q) wt(Q).*(h(Q)./psi(Q));
X = @(Q) sum(C(Q)) - sum(om);

A1 = @(Q) (om(1) - [1 0 0]*C(Q))./Q;
A2 = @(Q) (om(2) + A1(Q) - [0 1 0]*C(Q))./Q;
A = @(Q) (A1(Q) + A2(Q));

LQ = linspace(-2.5,2.5);
Q = exp(LQ);

figure(1)
tiledlayout(2,1)
nexttile
plot(LQ,-A(Q),'LineWidth',2)
hold on;
plot(LQ,LQ*0,'--k','LineWidth',1)
hold off
axis tight
ax = gca;
ax.XAxis.FontSize = 20;
ax.YAxis.FontSize = 20;
title(['  \eta = 1/' num2str(1/ET)  ...
    '  \lambda = ' num2str(Lambda), '  \beta = ' num2str(Beta) ])
ylabel('Net Asset Demand');
xlabel('Log(R)')
fig = gcf;
ax = fig.CurrentAxes;
line([0 0],ax.YLim,'Color','Red','LineWidth',1.5,...
    'LineStyle','--')
fontsize(gca,25,'pixels')

nexttile
plot(LQ,-X(Q),'LineWidth',2)
hold on;
plot(LQ,LQ*0,'--k','LineWidth',1)
hold off
axis tight
ax = gca;
ax.XAxis.FontSize = 20;
ax.YAxis.FontSize = 20;
title(['  \eta = 1/' num2str(1/ET)  ...
    '  \lambda = ' num2str(Lambda), '  \beta = ' num2str(Beta) ])
ylabel('Excess Demand for Goods');
xlabel('Log(R)')
fig = gcf;
ax = fig.CurrentAxes;
line([0 0],ax.YLim,'Color','Red','LineWidth',1.5,...
    'LineStyle','--')



fontsize(gca,25,'pixels')



