clear;clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
modelType = 'GuvenenProduction';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get preference parameters
[T,ET,TH,om,B,D,DEF,a,b] = pickParams(modelType);
om = om';
B = B';
N = 2*T-1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define key functions for the technology
Ffr = @(Q) 1./Q - 1 + D;
Ffk = @(Q) (TH./Ffr(Q)).^(1/(1-TH));
Ffy = @(Q) (Ffk(Q)).^TH;
Ffw = @(Q) (1-TH)*(TH./Ffr(Q)).^(TH/(1-TH));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define key functions for preferences
h   = @(Qp,w) sum((om.*w.*Qp));
psi = @(Qp) sum(((B.^ET).*(Qp.^(1-ET))));
wt  = @(Qp) ((B.^ET).*Qp.^(-ET));
C   = @(Qp,w) (wt(Qp).*h(Qp,w))./psi(Qp);



hPast  =  @(Qp,w) tril(ones(T))*(Qp.*om.*w);
hFutu  =  @(Qp,w) triu(ones(T))*(Qp.*om.*w);
CPast  =  @(Qp,w) (tril(ones(T))*(Qp.*C(Qp,w)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define steady state functions

hs = @(Q)  sum((Ffw(Q.*ones(T,1)).*om).*(Q.^((1:T)')));
psis = @(Q) sum((B.^ET).*Q.^((1:T)'*(1-ET)));
wts  = @(Q) (B.^ET).*((Q.^((1:T)'*-ET)));
Cs   = @(Q) wts(Q).*hs(Q)./psis(Q);

hPasts  =  @(Q) tril(ones(T))*((Q.^((1:T)')).*(om.*Ffw(Q.*ones(T,1))));
CPasts  =  @(Q) (tril(ones(T))*((Q.^((1:T)')).*Cs(Q)));

hFutus  =  @(Q) triu(ones(T))*((Q.^((1:T)')).*(om.*Ffw(Q.*ones(T,1))));
psis1 = @(Q) triu(ones(T))*((B.^ET).*Q.^((1:T)'*(1-ET)));

As      =  @(Q) [zeros(1,T);eye(T-1) zeros(T-1,1)]...
    *(hPasts(Q) - CPasts(Q))./(Q.^((0:T-1)'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Excess demand function for goods
Xs = @(Q) Ffy(Q)-sum(Cs(Q)) - D*Ffk(Q) - DEF;
% Excess demand function for assets
XAs = @(Q) (sum(As(Q)) - Ffk(Q));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LQ = linspace(a,b,1000);
Q  = exp(LQ);

Qa = Q(1:end-1);
Qb = Q(2:end);
Q_init = Q(sign(Xs(Qa)) ~= sign(Xs(Qb)));
Qs = uniquetol(fsolve(Xs,Q_init),1e-6);
disp(['Zeros of Excess Demand for Goods   : ' newline num2str(Qs,3) newline])
disp(['Government Debt at the Steady State: ' newline num2str(XAs(Qs),2) newline])

Qa_init = Q(sign(XAs(Qa)) ~= sign(XAs(Qb)));
Qas = uniquetol(fsolve(XAs,Qa_init),1e-1);
% disp(['Zeros of Exces Demand for Assets: ' newline num2str(Qas,3) newline])

close all
figure(1)
tiledlayout(2,1)

nexttile
plot(-LQ,Xs(1./Q),'LineWidth',2)
hold on
plot(LQ,Q*0,'k--','LineWidth',1)

fig = gcf;
ax = fig.CurrentAxes;
fontsize(gca,25,'pixels')



line([0 0],ax.YLim,'Color','Black','LineWidth',1.5,...
    'LineStyle','--')
hold off

fig = gcf;
ax = fig.CurrentAxes;
fontsize(gca,25,'pixels')


axis tight
xlim([-0.02,0.02])



nexttile
plot(-LQ,XAs(1./Q)./max(XAs(1./Q)),'LineWidth',2,'Color','Red')
hold on
plot(-LQ,Q*0,'k--','LineWidth',1)
fig = gcf;
ax = fig.CurrentAxes;
fontsize(gca,25,'pixels')
line([0 0],ax.YLim,'Color','Black','LineWidth',1.5,...
    'LineStyle','--')
hold off
axis tight

hold off
axis tight
xlim([-0.02,0.02])




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  This section used to debug the code
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% om = ones(T,1);
% % om = sym('om',[T,1],'real');
% B = ones(T,1);
% TH = sym('TH','real');
% assume(TH>0);
% assumeAlso(TH<1);
% 
% D = sym('D','real');
% ET = sym('ET','real');
% 
% D = 1;
% TH = 0;
% 
% Ffr = @(Q) 1./Q - 1 + D;
% Ffk = @(Q) (TH./Ffr(Q)).^(1/(1-TH));
% Ffy = @(Q) (Ffk(Q)).^TH;
% Ffw = @(Q) (1-TH)*(TH./Ffr(Q)).^(TH/(1-TH));
% 
% Qp = sym('Qp',[T,1],'real');
% w  = sym('w',[T,1],'real');
% Qs = sym('Q','real');
%  Q = sym('Q',[T,1],'real');
%  Qp = cumprod(Q);
%  assume(Q>0);
% assume(Qs>0);
% w = Ffw(Q);
% 
% J1 = @(Q) jacobian(h(Qp,Ffw(Q)),Q)
% J2 = @(Q) jacobian(Ffw(Q),Q)
% J3 = @(Q) jacobian(h(Qp,w),w)
% J4 = @(Q) jacobian(psi(Qp),Q)
% J5 = @(Q) jacobian(wt(Qp),Q)
% J6 = @(Q) jacobian(C(Qp,Ffw(Q)),Q)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the jacobian of the excess demand function

Jps = @(Q) (repmat(Q.^((0:T-1)'),1,T)).*tril(ones(T));

Ffr_Qs      = @(Q) diag(-1./(Q*ones(T,1)).^2);
Ffw_Qs      = @(Q) diag(TH^(1/(1-TH))*(1./(Q*ones(T,1))-1+D)...
    .^(1/(TH-1))./(Q*ones(T,1)).^2);

if TH ==0 || D == 1
    Ffk_Qs = @(Q) 0;
    Ffy_Qs = @(Q) 0;
else
    Ffk_Qs      = @(Q) diag((1/(1-TH))*((1./(Q)-1+D)/TH)...
        .^((2-TH)/(TH-1))./(TH*Q.^2));
    Ffy_Qs      = @(Q) TH*Ffk(Q).^(TH-1)*Ffk_Qs(Q);
end


h_Qs        = @(Q) (om.*Ffw(Q*ones(T,1))).'*Jps(Q) + (om.*(Q.^((1:T)'))).'*Ffw_Qs(Q);
psi_Qs      = @(Q) ((1-ET)*(B.^ET).*((Q.^((1:T)')).^-ET)).'*Jps(Q);
wt_Qs       = @(Q) -diag(ET*(wt((Q.^((1:T).')))./(Q.^((1:T).'))).')*Jps(Q);
Term_1      = @(Q)  hs(Q)./psis(Q);
Term_1_Q    = @(Q) (h_Qs(Q)/psis(Q)-hs(Q)*psi_Qs(Q)/psis(Q).^2);
C_Qs        = @(Q) wts(Q).*Term_1_Q(Q) + Term_1(Q).*wt_Qs(Q);

x1          = @(Q) [zeros(1,T-1) Ffk_Qs(Q) zeros(1,N-T)]; % Future capital;
x2          = @(Q) [zeros(1,T) Ffk_Qs(Q) zeros(1,N-T-1)]; % Current capital;
x3          = @(Q) [zeros(1,T-1) Ffy_Qs(Q) zeros(1,N-T)]; % Current GDP

n = numel(Qs);
X_Q = cell(n,1);
J1 = cell(n,1);
J2 = cell(n,1);
aevs = cell(n,1);
AA = cell(n,1);
BB = cell(n,1);
QQ = cell(n,1);
ZZ = cell(n,1);
nfstable = cell(n,1);
Ac = cell(n,1);
Cc = cell(n,1);
dCc = cell(n,1);
IndetNum = zeros(n,1);

fprintf(  '\n');
disp('______________________________________________________________')
for j = 1:n
    Ac{j} = zeros(T,1);
    X_Q{j} = zeros(1,N);
    J1{j} = sparse(zeros(N-1,N-1));
    J2{j} = sparse(zeros(N-1,N-1));
    S = zeros(1,N);
    temp1 = C_Qs(Qs(j));
    for t = 1:T
        temp = temp1(t,:);
        S(1,T-t+1:2*T-t) = S(1,T-t+1:2*T-t) + temp;
    end
    X_Q{j} = S + x2(Qs(j)) - (1-D)*x1(Qs(j)) - x3(Qs(j));
    J1{j}(1,:) = X_Q{j}(1,1:end-1);
    J1{j}(2:end,2:end) = sparse(eye(N-2));
    J2{j}(1,end) = - X_Q{j}(1,end);
    J2{j}(2:end,1:end-1) = sparse(eye(N-2));
    [AA{j},BB{j},QQ{j},ZZ{j}] = qz(full(J2{j}),full(J1{j}));
    aevs{j} = abs(diag(AA{j}./BB{j}));
    nfstable{j}  = sum(aevs{j}<1);
    
    
    disp(['Value of Excess Demand at Steady State where Q = ' ...
        num2str(Qs(j),'%.3f') ' is ' num2str(Xs(Qs(j)),'%.1f')])
    disp(['Value of Outside Asset Demand is ' ...
        num2str(XAs(Qs(j))*100,'%.2f') ' percent of GDP'])
    IndetNum(j) =  T-1 - nfstable{j};
    disp([ 'Degree of indeterminacy is ' num2str(IndetNum(j)) ])
    disp('______________________________________________________________')

    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions use to construct initial conditions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fP = @(Q) cumprod(Q);
% P as a function of Q
DPQ = @(Q) tril(ones(N).*fP(Q)./(Q'));
% Derivative of P w.r.t Q
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fw = @(Q) (1-TH)*((1./Q - 1 + D)./TH).^(TH/(TH-1));
% Wage as a function of Q
Dfw = @(Q) diag(TH.^(1/(1-TH)).*(1./Q-1+D).^(1/(TH-1))./Q.^2);
% Derivative of wage
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fk = @(Q)  ((1./Q - 1 + D)./TH).^(1/(TH-1));
% Capital as a function of Q
Dfk = @(Q) diag((1/(1-TH)).*((1./Q-1+D)/TH).^((2-TH)/(TH-1))./(Q.^2*TH));
% Derivative of k w.r.t Q
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fy = @(Q)  fk(Q).^TH;
% Capital as a function of Q
Dfy = @(Q) TH*fk(Q).^(TH-1).*Dfk(Q);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Omeg = @(Age,Date,Q) fw(Q).*[zeros(Date-1,1);om(Age:T);...
    zeros(N -T+Age- Date,1)]   ;
%  Values of endowment

DOmegQ = @(Age,Date,Q) Dfw(Q).*[zeros(Date-1,1);om(Age:T);...
    zeros(N -T+Age- Date,1)];
% Derivative of Omeg w.r.t Q
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fh = @(Age,Date,Q,P) Omeg(Age,Date,Q).'*[zeros(Date-1,1);P(Date:Date+T-Age,1);...
    zeros(N -T+Age- Date,1)];
% Human wealth
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psi = @(Age,Date,P) [zeros(Date-1,1);B(Age:T,1).^ET;...
    zeros(N -T+Age- Date,1)].'*[zeros(Date-1,1);P(Date:Date+T-Age,1)...
    .^(1-ET);zeros(N -T+Age- Date,1)];
% Denominator of Consumption weighting function

DpsiP = @(Age,Date,P) (1-ET).*[zeros(Date-1,1);...
    B(Age:T,1).^ET.*P(Date:Date+T-Age,1).^-ET;zeros(N -T+Age- Date,1)].';
% Derivative of psi w.r.t P
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phi = @(Age,Date,P) [zeros(Date-1,1);B(Age:T,1).^ET.*...
    P(Date:Date+T-Age,1).^-ET;zeros(N -T+Age- Date,1)];
% Vector of consumption weights

DphiP = @(Age,Date,P) diag([zeros(Date-1,1);-ET*B(Age:T,1).^ET.*...
    P(Date:Date+T-Age,1).^(-ET-1);zeros(N -T+Age- Date,1)]);
% Derivative of phi w.r.t phi
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Phi = @(Age,Date,P) phi(Age,Date,P)./psi(Age,Date,P);
% Vector of consumption weights

DPhiP = @(Age,Date,P) (psi(Age,Date,P).*DphiP(Age,Date,P) ...
    - phi(Age,Date,P).*DpsiP(Age,Date,P))./(psi(Age,Date,P).^2);
% Derivative of Phi w.r.t P
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DPhiQ = @(Age,Date,Q) DPhiP(Age,Date,fP(Q))*DPQ(Q);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DfhQ = @(Age,Date,Q,P) (DOmegQ(Age,Date,Q)*[zeros(Date-1,1);...
    P(Date:Date+T-Age,1);zeros(N -T+Age- Date,1)]).';
% Derivative of h w.r.t Q

DfhP = @(Age,Date,Q) Omeg(Age,Date,Q).';
% Derivateive of h w.r.t P

Dfh = @(Age,Date,Q) DfhQ(Age,Date,Q,fP(Q)) + DfhP(Age,Date,Q)*DPQ(Q);
% Total derivative of h w.r.t 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fC = @(Age,Date,Q,A)  (fh(Age,Date,Q,fP(Q))+A)* Phi(Age,Date,fP(Q));
% Consumption

DfCQ = @(Age,Date,Q,A) Dfh(Age,Date,Q).*(Phi(Age,Date,fP(Q))) + ...
    DPhiQ(Age,Date,Q)*(fh(Age,Date,Q,fP(Q))+A);
% Derivative of Consumption w.r.t Q

DfCA = @(Age,Date,Q) Phi(Age,Date,fP(Q));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct Initial Conditions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
 g = cell(n,1);
 Av = cell(n,1);
 FXs = cell(n,1);
 DFXsQ = cell(n,1);
 DFXsA = cell(n,1);
 A = cell(n,1);
 Debt = cell(n,1); 

 for j = 1:n
     g{j} = zeros(N,1);
     FXs{j} = zeros(T,1);
     DFXsQ{j} = zeros(N,N);
     DFXsA{j} = zeros(N,1);
     Q = ones(N,1)*Qs(j);
     for age = 1:T
         A{j} = As(Qs(j));
         g{j} = g{j} + fC(age,1,Q,A{j}(age));
         DFXsQ{j} = DFXsQ{j} + DfCQ(age,1,Q,A{j}(age));
         DFXsA{j} = DFXsA{j} + DfCA(age,1,Q);
     end
     for t = 2:T
         g{j} = g{j} + fC(1,t,Q,0);
         DFXsQ{j} = DFXsQ{j} + DfCQ(1,t,Q,0);
         DFXsA{j} = DFXsA{j} + DfCA(1,t,Q);
     end
     FXs{j} = g{j} + D*Ffk(Q) - Ffy(Q);
     if D==1
         DFXsQ{j} = DFXsQ{j} ;
         DFXsA{j} = DFXsA{j} ;
     else
         DFXsQ{j} = DFXsQ{j} + D*Dfk(Q) - Dfy(Q);
         DFXsA{j} = DFXsA{j};
     end
     FXs{j} = FXs{j}(1:T);
     DFXsQ{j} = DFXsQ{j}(1:T,:);
     DFXsA{j} = DFXsA{j}(1:T,:);
     Debt{j} = sum(A{j}) - Ffk(Qs(j));
 end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute Dynamics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
j = find(abs(Qs-1) < 1e-4);     % Select the golden-rule steady state


n1 = nfstable{j};   % Number of forward stable roots
n2 = N-1-n1;        % Number backward stable roots

% Decompose the matrices AA and BB into blocks associated with the stable
% and unstable generalized eigenvalues of {J1,J2}
A11 = AA{j}(1:n2,1:n2);
B11 = BB{j}(1:n2,1:n2);
A22 = AA{j}(n2+1:end,n2+1:end);
B22 = AA{j}(n2+1:end,n2+1:end);

Z  = ZZ{j};         % Used to partition Y into stable and unstable blocks
Z1 = Z(:,1:n2);     
Z2 = Z(:,n2+1:end);

Z11 = Z(1:n2,1:n2);
Z12 = Z(1:n2,n2+1:end);
Z21 = Z(n2+1:end,1:n2);
Z22 = Z(n2+1:end,n2+1:end);

A_Init = A{j};  % Initial asset values associated with the golden-rule 
                % steady state
    
MB = A11\B11;   % Matrix of coefficents for the reduced form which takes 
                % the form:  Y_(t+1) = MB{j} x Y_(t)

X   = X_Q{j};   % Excess demand function for initial periods              

DFQ2 = DFXsQ{j};
DFQ1 = DFQ2(:,1:end-1) - ...         % Partial derivative of 
    DFQ2(:,end)*X(1:end-1)/X(end);   % excess demand function w.r.t Q  

DFA  = DFXsA{j};   % Partial derivative of excess demand function w.r.t 
                    % initial wealth: NB price effects work through initial
                    % wealth vector, 'A' and trhough debt
                        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code computes the initial conditions in periods 1 through T using
% the GOODS MARKET equilibrium conditions as opposed to the asset market
% equilibrium conditions. The two approaches should yield identical
% results. Goods market equilim conditions are more staightforward.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct matrices R and r such that R x DFX = r, where R is n2 x n2, 
% square and invertible and n2 is the number of stable roots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p_shock = 0.00;       % Percentage shock to initial price level
                      % Set this to zero to simulate a real rate shock with
                      % no contempraneous price reponse
r_shock = 0.01;       % Percentage shock to period 2 interest rate
                      % Set this to zero to siulate a price level shock
                      % with no independt shock to period 2 interest rate
r1 = - DFA.*A_Init...   % DFA is the vector of derivatives of excess demands
    .*p_shock;          % w.r.t initial wealth. A_Init weights these percentage
                        % changes by initial wealth



Sunspot_Assumption = zeros(1,N-1);
Sunspot_Assumption(2) = 1;      % This assumption forces the price shock to Q_2
Sunspot_Assumption(T) = ...     % to equal the price shock to Q_T. This assumption
    -1*(r_shock==0);            % kicks in only if there is no interest rate shock.
                                % when there is a non-zero interest rate shock
                                % the shock hits ONLY the interest rate in period 2 
                                
 
R = [DFQ1;Sunspot_Assumption];     % Matrix of restrictions on initial 
                                   % conditions Y_Init

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Combine restrictions on X with zero restrictions on unstable elements of Y
R1 = R(:,1:n2);     
R2 = R(:,n2+1:end);
r = [r1;r_shock];
X1 = (R1 + R2*Z21*Z11^-1)^-1*r;
Y1 = Z11\X1;
X2 = Z21*Z11^-1*X1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute effect of a price level shock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X_Init = real([X1;X2]); % X is an N x 1 vector of deviations of one-step ...
                        % ahead interest rates from the steady state.
                        % X_Init contaons the initial values of X

Y_Init = zeros(N-1,1);  % Y = Z'X is a linear combinations of the X's used 
                        % to diagonalize the system. NB. J2 DX' = J1 DX 
                        % where J2 = Q[AA]Z' and J1 = Q[BB]Z'. Y_Init contains
                        % the initial values of Y

Y_Init(1:n2) = Y1;

                        
N_Sims = 200;

Y = zeros(N-1,N_Sims);
Y(1:n2,1) = Y_Init(1:n2);
Dates = 1:N_Sims;

for t = 2:N_Sims
    Y(1:n2,t) = MB*Y(1:n2,t-1);
end

X = real(Z*Y);                
  
figure(1)
tiledlayout(2,1)
nexttile
plot(Dates,-100*X(1,:),'LineWidth',2)
hold on
plot(0*Dates,'--k','LineWidth',1)
hold off
xlabel('Years','FontSize',18)
ylabel(['Percent'],'FontSize',18)
title('The Real Interest Rate','FontSize',20)
ax = gca;
ax.XAxis.FontSize = 20;
ax.YAxis.FontSize = 20;


nexttile
plot(Ffk(1+X(1,:)),'r-o','LineWidth',2,'MarkerIndices',1:2:length(Ffk(1+X(1,:))))
hold on
plot(ones(1,N_Sims)*Ffk(1),'--k','LineWidth',1)
hold off
xlabel('Years','FontSize',18)
ylabel('Fraction of GDP','FontSize',18)

title('The Capital Stock','FontSize',20)
ax = gca;
ax.XAxis.FontSize = 20;
ax.YAxis.FontSize = 20;

% disp('Complete')

